<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2024 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\Console\Command;

use InvalidArgumentException;
use JsonException;
use Threema\Console\Common;
use Threema\MsgApi\Connection;
use Threema\MsgApi\ConnectionSettings;
use Threema\MsgApi\Core\Exception;
use Threema\MsgApi\Helpers\E2EHelper;
use Threema\MsgApi\PublicKeyStore;

class GroupSendE2EText extends Base
{
	public function __construct(private readonly PublicKeyStore $publicKeyStore)
	{
		parent::__construct(
			'Send End-to-End Encrypted Text Message to a group',
			[self::argThreemaIds, self::argFrom, self::argSecret, self::argPrivateKey, self::argGroupId],
			'Encrypt standard input and send the text message to the given ID. \'from\' is the API identity and \'secret\' is the API secret. This prints the message ID on success.',
			[self::argQuotedMessageId]
		);
	}

	/**
	 * @throws Exception
	 * @throws JsonException
	 */
	public function doRun()
	{
		$threemaIds = $this->getArgumentThreemaIds(self::argThreemaIds);
		$from = $this->getArgument(self::argFrom);
		$secret = $this->getArgument(self::argSecret);
		$privateKey = $this->getArgumentPrivateKey(self::argPrivateKey);
		$groupId = $this->getArgumentGroupId(self::argGroupId);
		$messageId = $this->getArgument(self::argQuotedMessageId);

		Common::required($threemaIds, $from, $secret, $privateKey, $groupId);

		$message = $this->readStdIn();
		if ($message === '') {
			throw new InvalidArgumentException('please define a group message');
		}

		$settings = new ConnectionSettings(
			$from,
			$secret
		);

		$connector = new Connection($settings, $this->publicKeyStore);

		$helper = new E2EHelper($privateKey, $connector);
		$results = $helper->sendGroupTextMessages($groupId, $threemaIds, $message, $messageId);

		if (null === $results->getResults() || count($results->getResults()) === 0) {
			throw new Exception($results->getErrorMessage());
		}

		foreach ($results->getResults() as $threemaId => $result) {
			if ($result->isSuccess()) {
				Common::l("{$threemaId} Message ID : {$result->getMessageId()}");
			} else {
				Common::e("{$threemaId} Error: {$result->getErrorMessage()}");
			}
		}
	}
}
