<?php
/**
 * @author Threema Gateway team
 * @copyright Copyright (c) 2024 Threema GmbH
 * @see https://gateway.threema.ch/en/developer
 */

namespace Threema\MsgApi;

class ConnectionSettings
{
	final public const tlsOptionForceHttps = 'forceHttps';
	final public const tlsOptionVersion = 'tlsVersion';
	final public const tlsOptionCipher = 'tlsCipher';

	private array $tlsOptions = [];
	private ?string $userAgent;

	/**
	 * @param string $threemaId valid threema id (8 chars)
	 * @param string $secret secret
	 * @param string|null $host server url
	 * @param array|null $tlsOptions advanced TLS options
	 */
	public function __construct(
		private readonly string $threemaId,
		private readonly string $secret,
		private readonly ?string $host = 'https://msgapi.threema.ch',
		?array $tlsOptions = null
	)
	{
		$userAgent = 'threema-msgapi-sdk-php/';
		$version = \Composer\InstalledVersions::getPrettyVersion('threema/msgapi-sdk');
		if (!empty($version)) {
			$userAgent .= $version;
		}
		$this->setUserAgent($userAgent);

		// TLS options
		if (null !== $tlsOptions) {
			if (true === array_key_exists(self::tlsOptionForceHttps, $tlsOptions)) {
				$this->tlsOptions[self::tlsOptionForceHttps] = true === $tlsOptions[self::tlsOptionForceHttps];
			}

			if (true === array_key_exists(self::tlsOptionVersion, $tlsOptions)) {
				$this->tlsOptions[self::tlsOptionVersion] = $tlsOptions[self::tlsOptionVersion];
			}

			if (true === array_key_exists(self::tlsOptionCipher, $tlsOptions)) {
				$this->tlsOptions[self::tlsOptionCipher] = $tlsOptions[self::tlsOptionCipher];
			}
		}
	}

	public function getThreemaId(): string
	{
		return $this->threemaId;
	}

	public function getSecret(): string
	{
		return $this->secret;
	}

	public function getHost(): ?string
	{
		return $this->host;
	}

	public function getUserAgent(): ?string
	{
		return $this->userAgent;
	}

	public function setUserAgent(?string $userAgent): void
	{
		$this->userAgent = $userAgent;
	}

	public function getTlsOptions(): array
	{
		return $this->tlsOptions;
	}

	public function getTlsOption(mixed $option, mixed $default = null): ?string
	{
		return true === array_key_exists($option, $this->tlsOptions) ? $this->tlsOptions[$option] : $default;
	}
}
